/* This file was generated by syncqt. */
#ifndef QT_QTSCXML_VERSION_H
#define QT_QTSCXML_VERSION_H

#define QTSCXML_VERSION_STR "5.9.5"

#define QTSCXML_VERSION 0x050905

#endif // QT_QTSCXML_VERSION_H
