/* This file was generated by syncqt. */
#ifndef QT_QTFONTDATABASESUPPORT_VERSION_H
#define QT_QTFONTDATABASESUPPORT_VERSION_H

#define QTFONTDATABASESUPPORT_VERSION_STR "5.9.5"

#define QTFONTDATABASESUPPORT_VERSION 0x050905

#endif // QT_QTFONTDATABASESUPPORT_VERSION_H
