/* This file was generated by syncqt. */
#ifndef QT_QTPLATFORMHEADERS_VERSION_H
#define QT_QTPLATFORMHEADERS_VERSION_H

#define QTPLATFORMHEADERS_VERSION_STR "5.9.5"

#define QTPLATFORMHEADERS_VERSION 0x050905

#endif // QT_QTPLATFORMHEADERS_VERSION_H
