/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the QtCore module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or (at your option) the GNU General
** Public license version 3 or any later version approved by the KDE Free
** Qt Foundation. The licenses are as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-2.0.html and
** https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef QSYSTEMERROR_P_H
#define QSYSTEMERROR_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include <QtCore/private/qglobal_p.h>
#include <qstring.h>

QT_BEGIN_NAMESPACE

class Q_CORE_EXPORT QSystemError
{
public:
    enum ErrorScope
    {
        NoError,
        StandardLibraryError,
        NativeError
    };

    inline QSystemError(int error, ErrorScope scope);
    inline QSystemError();

    inline QString toString() const;
    inline ErrorScope scope() const;
    inline int error() const;

    static QString string(ErrorScope errorScope, int errorCode);
    static QString stdString(int errorCode = -1);
#ifdef Q_OS_WIN
    static QString windowsString(int errorCode = -1);
#endif

    //data members
    int errorCode;
    ErrorScope errorScope;
};

QSystemError::QSystemError(int error, QSystemError::ErrorScope scope)
: errorCode(error), errorScope(scope)
{

}

QSystemError::QSystemError()
: errorCode(0), errorScope(NoError)
{

}

QString QSystemError::toString() const
{
    return string(errorScope, errorCode);
}

QSystemError::ErrorScope QSystemError::scope() const
{
    return errorScope;
}

int QSystemError::error() const
{
    return errorCode;
}

QT_END_NAMESPACE

#endif // QSYSTEMERROR_P_H
