/* This file was generated by syncqt. */
#ifndef QT_QT3DANIMATION_VERSION_H
#define QT_QT3DANIMATION_VERSION_H

#define QT3DANIMATION_VERSION_STR "5.9.5"

#define QT3DANIMATION_VERSION 0x050905

#endif // QT_QT3DANIMATION_VERSION_H
